@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #
-# $Header: /Users/claude/fuzz/lib/Genezzo/RCS/GenDBI.pm,v 6.3 2004/08/24 21:34:39 claude Exp claude $
+# $Header: /Users/claude/fuzz/lib/Genezzo/RCS/GenDBI.pm,v 6.6 2004/09/11 06:59:33 claude Exp claude $
 #
 # copyright (c) 2003, 2004 Jeffrey I Cohen, all rights reserved, worldwide
 #
@@ -48,11 +48,11 @@ BEGIN {
 	
 }
 
-our $VERSION   = '0.22';
+our $VERSION   = '0.23';
 our $RELSTATUS = 'Alpha'; # release status
 # grab the code check-in date and convert to YYYYMMDD
 our $RELDATE   = 
-    do { my @r = (q$Date: 2004/08/24 21:34:39 $ =~ m|Date:(\s+)(\d+)/(\d+)/(\d+)|); sprintf ("%04d%02d%02d", $r[1],$r[2],$r[3]); };
+    do { my @r = (q$Date: 2004/09/11 06:59:33 $ =~ m|Date:(\s+)(\d+)/(\d+)/(\d+)|); sprintf ("%04d%02d%02d", $r[1],$r[2],$r[3]); };
 
 # Preloaded methods go here.
 
@@ -1474,20 +1474,21 @@ sub SQLAlter
                            return 0;
                        }
 
-                       my $stat = 
+                       my ($stat, $new_consname, $new_iname) = 
                            $dictobj->DictTableAddConstraint(%nargs);
+
                        my $outstr;
                        if ($stat)
                        {
-                           $cons_name = ""
+                           $cons_name = $new_consname
                                unless (defined($cons_name));
                            $outstr = "Added constraint $cons_name" .
                                " to table $tablename";
                        }
                        else
                        {
-                               $outstr = "Failed to add constraint";
-                           }
+                           $outstr = "Failed to add constraint";
+                       }
                        print $outstr, "\n";
                        return $stat;
                    } # end if exists constraint
@@ -1511,7 +1512,7 @@ sub SQLUpdate
     return undef
         if ($badparse);
 
-    greet $sql_cmd;
+#    greet $sql_cmd;
 
     my $tablename = $sql_cmd->{tablename};
 
